@@ -1,12 +1,11 @@
 import monocle
 from monocle import _o, Return
 from monocle.callback import Callback, defer
-
+from monocle.stack import eventloop as evlp
 
 class ConnectionLost(Exception):
     pass
 
-
 # Connection takes a stack_conn, which should have this interface:
 #
 # def write(data):
@@ -31,19 +30,46 @@ class ConnectionLost(Exception):
 #
 # buffer: string buffer which grows before read_cb is called
 
+
 class Connection(object):
     def __init__(self, stack_conn=None):
         self._stack_conn = stack_conn
         self.writing = False
-        self._flush_cb = Callback()
+        self.flush_cb = Callback()
         self.write_encoding = 'utf-8'
+        self.timeout = None
+        self._current_timeout = None
+
+    def _time_out(self, cb, timeout):
+        self._current_timeout = None
+        if hasattr(cb, 'result'):
+            return
+        if cb == self._stack_conn.read_cb:
+            self._stack_conn.read_cb = None
+        elif cb == self.flush_cb:
+            self.flush_cb = None
+        elif cb == self._stack_conn.connect_cb:
+            self._stack_conn.connect_cb = None
+        self._stack_conn.disconnect()
+        cb(ConnectionLost("connection timed out after %s seconds" % timeout))
+
+    def _queue_timeout(self, cb):
+        if self.timeout is not None:
+            if self._current_timeout:
+                self._current_timeout.cancel()
+            self._current_timeout = evlp.queue_task(self.timeout,
+                                                    self._time_out,
+                                                    cb,
+                                                    self.timeout)
 
     @_o
     def read_some(self):
         self._check_reading()
+
         if not self._stack_conn.buffer:
             self._check_closed()
             self._stack_conn.resume()
+            self._queue_timeout(self._stack_conn.read_cb)
             yield self._stack_conn.read_cb
         tmp = self._stack_conn.buffer
         self._stack_conn.buffer = ""
@@ -52,9 +78,11 @@ class Connection(object):
     @_o
     def read(self, size):
         self._check_reading()
+
         while len(self._stack_conn.buffer) < size:
             self._check_closed()
             self._stack_conn.resume()
+            self._queue_timeout(self._stack_conn.read_cb)
             yield self._stack_conn.read_cb
         tmp = self._stack_conn.buffer[:size]
         self._stack_conn.buffer = self._stack_conn.buffer[size:]
@@ -63,6 +91,7 @@ class Connection(object):
     @_o
     def read_until(self, s):
         self._check_reading()
+
         while True:
             size = self._stack_conn.buffer.find(s)
             if size != -1:
@@ -70,6 +99,7 @@ class Connection(object):
                 break
             self._check_closed()
             self._stack_conn.resume()
+            self._queue_timeout(self._stack_conn.read_cb)
             yield self._stack_conn.read_cb
         tmp = self._stack_conn.buffer[:size]
         self._stack_conn.buffer = self._stack_conn.buffer[size:]
@@ -87,22 +117,27 @@ class Connection(object):
             return defer(None)
         self.writing = True
         self._stack_conn.write(data)
-        return self.flush()
+        cb = self.flush()
+        self._queue_timeout(cb)
+        return cb
 
     def _write_flushed(self, result=None):
         self.writing = False
-        cb = self._flush_cb
-        self._flush_cb = Callback()
+        cb = self.flush_cb
+        self.flush_cb = Callback()
         cb(result)
 
     def flush(self):
         self._check_closed()
-        cb = self._flush_cb
+        cb = self.flush_cb
         if not self.writing:
             self._write_flushed()
         return cb
 
     def _closed(self, reason):
+        if self._current_timeout:
+            self._current_timeout.cancel()
+            self._current_timeout = None
         cl = ConnectionLost(str(reason))
         cl.original = reason
         if self._stack_conn.connect_cb:
