@@ -5,19 +5,17 @@
 import errno
 import socket
 import new
-import time
 
 try:
-    import ssl # Python 2.6+
+    import ssl  # Python 2.6+
 except ImportError:
     ssl = None
 
-import tornado.ioloop
 from tornado.iostream import IOStream, SSLIOStream
 
 from monocle import _o, launch
 from monocle.callback import Callback
-from monocle.stack.network import Connection, ConnectionLost
+from monocle.stack.network import Connection
 from monocle.tornado_stack.eventloop import evlp
 
 
@@ -145,15 +143,21 @@ class TornadoConnection(Connection):
 
     def read_some(self):
         self._check_reading()
-        return self._stack_conn.read_some()
+        cb = self._stack_conn.read_some()
+        self._queue_timeout(cb)
+        return cb
 
     def read(self, size):
         self._check_reading()
-        return self._stack_conn.read(size)
+        cb = self._stack_conn.read(size)
+        self._queue_timeout(cb)
+        return cb
 
     def read_until(self, s):
         self._check_reading()
-        return self._stack_conn.read_until(s)
+        cb = self._stack_conn.read_until(s)
+        self._queue_timeout(cb)
+        return cb
 
 
 class Service(object):
@@ -238,7 +242,7 @@ class Client(TornadoConnection):
         self.ssl_options = None
 
     @_o
-    def connect(self, host, port, timeout=30):
+    def connect(self, host, port):
         s = socket.socket()
         if self.ssl_options is not None:
             iostream = SSLIOStream(s, ssl_options=self.ssl_options)
@@ -248,16 +252,8 @@ class Client(TornadoConnection):
         self._stack_conn.attach(self)
         self._stack_conn.connect((host, port))
 
-        if timeout is not None:
-            cb = self._stack_conn.connect_cb
-            def _on_timeout():
-                if hasattr(cb, 'result'):
-                    return
-                self._stack_conn.connect_cb = None
-                self._stack_conn.disconnect()
-                cb(ConnectionLost("connection timed out after %s seconds" % timeout))
-            self._timeout = iostream.io_loop.add_timeout(time.time() + timeout,
-                                                         _on_timeout)
+        if self.timeout is not None:
+            self._queue_timeout(self._stack_conn.connect_cb)
 
         yield self._stack_conn.connect_cb
 
